/*
 DSPAM
 COPYRIGHT (C) 2004 NETWORK DWEEBS CORPORATION

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifdef HAVE_CONFIG_H
#include <auto-config.h>
#endif

#include "lht.h"
#include "nodetree.h"
#include "error.h"
#include "storage_driver.h"
#include "decode.h"
#include "libdspam_objects.h"
#include "util.h"
#include "tbt.h"

#ifndef _LIBDSPAM_H
#  define _LIBDSPAM_H

/*
  libdspam public functions 

  dspam_init()
    Initialize a DSPAM context.  This function allocates space for a new
    context and configures all of the basic operating parameters.

  dspam_process()
    Processes the provided message according to the operating parameters
    specified in the context.  The message variable can be passed in as NULL,
    provided that the context's message variable has already been set to
    a pointer to an actualized structure of the message.  Otherwise, the
    text copy of the message will be converted to an actualized structure by
    libdspam.

  dspam_destroy()
    Destroys a DSPAM context and closes all open storage resources.

  dspam_getsource()
    Traverses the message headers and copies the source address of the message
    into the supplied buffer.
*/

DSPAM_CTX * dspam_init	(const char *username, 
			 const char *group,
			 int operating_mode,
			 u_int32_t flags);

int dspam_process	(DSPAM_CTX * CTX, const char *message);
int dspam_destroy	(DSPAM_CTX * CTX);

int dspam_getsource	(DSPAM_CTX * CTX, 
                         char *buf, 
                         size_t size,
                         const char *whitelist);

/*
  libdspam internal functions 

  _ds_calc_stat()
    Performs a statistical analysis of the token referenced in the
    passed structure, and sets the structure's probability variable.
    This should be called once for every token stat loaded, when the
    statistical probability of the token is necessary to the operation.

  _ds_operate()
    libdspam's internal operating subroutine

  _ds_process_header_token()
    Performs analysis and composition on a single token in the message header

  _ds_process_body_token()
    Performs analysis and composition on a single token in the message body

  _ds_process_signature()
    Performs a training process on the signtuare provided in the filter context

*/

int _ds_calc_stat		(DSPAM_CTX * CTX, 
				 unsigned long long token,
				 struct _ds_spam_stat *);

int _ds_calc_result		(DSPAM_CTX * CTX,
                                 struct tbt *index,
                                 struct lht *freq);

int _ds_operate			(DSPAM_CTX * CTX, char *headers, char *body);

int _ds_process_header_token	(DSPAM_CTX * CTX, 
				 char *joined_token,
				 const char *previous_token,
				 struct lht *tokens,
				 const char *heading);

int _ds_process_body_token	(DSPAM_CTX * CTX,
				 char *joined_token,
				 const char *previous_token,
				 struct lht *tokens);

int _ds_process_signature	(DSPAM_CTX * CTX);

int _ds_analyze_message		(DSPAM_CTX *CTX,
                                 buffer *header, 
                                 buffer *body);

/* Return Codes */

#ifndef EINVAL
#       define EINVAL                   -0x01
  /* Invalid call or parms - already initialized or shutdown */
#endif
#define EUNKNOWN                -0x02   /* Unknown/unexpected error */
#define EFILE                   -0x03   /* File error */
#define ELOCK                   -0x04   /* Lock error */
#define EFAILURE                -0x05   /* Failed to perform operation */

#define FALSE_POSITIVE(A)	(A->classification == DSR_ISINNOCENT && A->source == DSS_ERROR)
#define SPAM_MISS(A)		(A->classification == DSR_ISSPAM && A->source == DSS_ERROR)
#define SIGNATURE_NULL(A)	(A->signature == NULL)

#endif /* _LIBDSPAM_H */

